"""This file contains some defaults for the django worker"""

STEPS = {
    'run': {
        'info': 'Run all steps',
        'steps': ['prepare', 'requirement', 'setupdb', 'project', 'nginx'],
    },
    'prepare': {
        'info': 'Prepare the system',
        'commands': [
            'sudo apt update -y',
            'sudo apt upgrade -y',
            'sudo apt install -y python3-dev python3-pip python3-venv',
            'sudo apt autoremove -y',
            'sudo -H pip3 install --upgrade pip',
            'cd {{ folder }}',
            'python3 -m venv venv',
            'source venv/bin/activate',
            'pip install --upgrade pip wheel django python-decouple',
            'deactivate',
        ]
    },
    'postgresql': {
        'info': 'Setup a postgresql DB.',
        'template': 'postgresql.template'
    },
    'project': {
        'info': 'Setupthe project, migrate db and collectstatic',
        'template': 'project.template'
    },
    'setupdb': {
        'info': 'Setup the DB.',
        'func': 'setup_db',
    },
    'requirement': {
        'info': 'Install the project requirements.',
        'func': 'install_requirement',
    },
    'nginx': {
        'info': 'setup everything for nginx',
        'steps': ['nginx-install', 'nginx-file', 'nginx-setup']
    },
    'nginx-install': {
        'info': 'Install nginx',
        'commands': ['sudo apt -y install nginx'],
    },
    'nginx-file': {
        'info': 'Create the nginx config file',
        'template': 'nginx-config.template',
        'filename': '/etc/nginx/sites-available/{{ project }}',
        'sudo': True,
    },
    'nginx-setup': {
        'info': 'Compile the config file and start nginx',
        'template': 'nginx-setup.template',
    },
    'gunicorn': {
        'steps': ['gunicorn-install', 'gunicorn-file', 'gunicorn-setup'],
    },
    'gunicorn-install': {
        'template': 'gunicorn-install.template',
    },
    'gunicorn-file': {
        'template': 'gunicorn-file.template',
        'filename': '{{ folder }}/gunicorn_start',
    },
    'gunicorn-setup': {
        'commands': [
            'chmod u+x {{ folder }}/gunicorn_start',
            'mkdir {{ folder }}/run',
            'mkdir {{ folder }}/logs',
            'touch {{ folder }}/logs/gunicorn.log'
        ],
    },
}



from axju.core.tools import setup_parser, setup_logger
from axju.worker.django import DjangoWorker
from axju.worker.git import GitWorker

def main():
    parser = setup_parser()
    subparsers = parser.add_subparsers(help='commands', dest='action')

    parser_git = subparsers.add_parser('git', help='coming soon')

    parser_django = subparsers.add_parser('django', help='Django deploying :)')
    parser_django.add_argument('--folder', type=str, default='.', help='The django project folder.')
    parser_django.add_argument('--setting', type=str, default='', help='Change the default the settings.')

    args = parser.parse_args()
    setup_logger(args.verbose, args.log)

    if args.action == 'django':
        worker = DjangoWorker(folder=args.folder, setting=args.setting)
    elif args.action == 'git':
        worker = GitWorker()
    else:
        return parser.print_help()

    if args.show:
        return worker.show(args.show)

    for step in args.steps:
        worker.run(step)
        

if __name__ == '__main__':
    main()



import os, sys
import logging
import subprocess
import tempfile
from jinja2 import Environment, BaseLoader, PackageLoader


class BasicWorker(object):
    """The basic worker"""

    def __init__(self, steps, templates):
        self.logger = logging.getLogger(self.__class__.__name__)
        self.logger.debug('create the object')

        self.steps = steps
        templateLoader = PackageLoader(*templates)
        self.templateEnv = Environment(loader=templateLoader)

    def execute(self, commands):
        """Connect to the shell and execute multiple commands. It is necessary
        to activate a virtual environment and then install some packages."""

        cmd = ['/bin/bash', '-i']
        encoding = 'utf-8'
        if sys.platform[:3] == 'win':
            cmd = ['cmd']
            encoding = 'cp437'
        p = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)

        for command in commands:
            self.logger.info('run command "%s"', command)
            p.stdin.write(str(command + "\n").encode())
            p.stdin.flush()

        p.stdin.close()
        self.logger.info('working...')
        self.logger.debug('result:\n%s', p.stdout.read().decode(encoding))
        p.wait()

    def render_data(self):
        return {
            'user': os.getlogin(),
        }

    def render_template(self, filename):
        template = self.templateEnv.get_template(filename)
        return template.render(self.render_data())

    def render_str(self, s):
        template = Environment(loader=BaseLoader()).from_string(s)
        return template.render(self.render_data())

    def render_commands(self, commands):
        """Render a list of commands. That can contain some dummy."""
        return [ self.render_str(c) for c in commands ]

    def show(self, kind='all'):
        """Display some infos about the steps."""
        n = max([ len(s) for s in self.steps ])
        for step, data in self.steps.items():
            print(step.ljust(n+1, '.'), data.get('info', 'missing'), sep='')

    def run(self, name):
        """Run the one step"""
        if name not in self.steps:
            self.logger.warning('No step definition for "%s"', name)
            return

        step = self.steps[name]
        self.logger.info('run step "%s"', name)

        if 'commands' in step:
            self.execute(self.render_commands(step['commands']))

        elif 'func' in step:
            getattr(self, step['func'])(**step.get('kwargs', {}))

        elif 'template' in step:
            t = self.render_template(step['template'])

            if 'filename' in step:
                filename = self.render_str(step['filename'])

                with tempfile.NamedTemporaryFile() as f:
                    f.write(t.encode())
                    f.flush()
                    if step.get('sudo', False):
                        subprocess.call(['sudo', 'cp', f.name, filename])
                    else:
                        subprocess.call(['cp', f.name, filename])

            else:
                self.execute(t.split('\n'))

        elif 'steps' in step:
            for s in step['steps']:
                self.run(s)

        self.logger.info('finished step "%s"', name)



import unittest

from axju.worker.django import DjangoWorker

class TestTemplate(unittest.TestCase):

    def test_template_project(self):
        worker = DjangoWorker(
            folder='./tests/django/projects/project1',
        )
        s1 = worker.render_template('test-project.template')
        s2 = '{}{}{}'.format(worker.project, worker.folder, worker.settings.__name__)
        self.assertEqual(s1,s2)

    def test_template_host1(self):
        worker = DjangoWorker(
            folder='./tests/django/projects/project1',
        )
        s = worker.render_template('test-host.template')
        self.assertEqual(s, 'test.de')

    def test_template_host2(self):
        worker = DjangoWorker(
            folder='./tests/django/projects/project2',
        )
        s = worker.render_template('test-host.template')
        self.assertEqual(s, 'test.de')

if __name__ == '__main__':
    unittest.main()



from axju.__about__ import (
    __author__, __copyright__, __email__, __license__, __summary__, __title__,
    __url__, __version__
)

__all__ = [
    '__title__', '__summary__', '__url__', '__version__', '__author__',
    '__email__', '__license__', '__copyright__',
]



__all__ = [
    '__title__', '__summary__', '__url__', '__version__', '__author__',
    '__email__', '__license__', '__copyright__',
]

__title__ = 'axju'
__summary__ = 'This is my personal package'
__url__ = 'https://github.com/axju/axju'

__version__ = '0.0.3a1'

__author__ = 'Axel Juraske'
__email__ = 'axel.juraske@short-report.de'

__license__ = 'MIT'
__copyright__ = 'Copyright 2018 %s' % __author__



import os, sys
import importlib.util

from axju.core.basic import BasicWorker
from axju.worker.django.settings import STEPS

class DjangoWorker(BasicWorker):
    """This class load the django settings"""

    APP_PIP = {
        'crispy_forms': 'django-crispy-forms',
        'debug_toolbar': 'django-debug-toolbar',
    }

    DEFAULT_SETTINGS = [
        '', 'development', 'production'
    ]

    def __init__(self, folder='.', setting=''):
        super(DjangoWorker, self).__init__(steps=STEPS, templates=('axju', 'worker/django/templates/'))

        print( folder, setting)

        self.folder = os.path.abspath(folder)
        self.project = os.path.basename(self.folder)

        sys.path.append(self.folder)
        self.settings = self.__load_settings(setting)

    def __str__(self):
        return '>>{}<< {}'.format(self.project, self.folder)

    def render_data(self):
        data = super(DjangoWorker, self).render_data()

        data['folder'] = self.folder
        data['project'] = self.project

        if self.settings:
            data['setting'] = self.settings.__name__
            data['settings'] = self.settings

            if getattr(self.settings, 'DATABASES', None):
                data['db'] = self.settings.DATABASES.get('default')

            if getattr(self.settings, 'HOST', None):
                data['host'] = self.settings.HOST

            if 'host' not in data and getattr(self.settings, 'ALLOWED_HOSTS', None):
                data['host'] = self.settings.ALLOWED_HOSTS[0]

        return data

    def __load_settings(self, setting):
        """Load the django settings. For single module settings == '', overwise
        mean multiple settings. First check if the setting exist. If not,try to
        load the default settings."""

        settings_list = self.find_settings()
        setting_name = setting

        if setting not in settings_list:
            self.logger.debug('No settings named "%s"', setting)

            for default in self.DEFAULT_SETTINGS:
                if default in settings_list:
                    setting_name = default
                    break

        s = '{}.settings'.format(self.project)
        if setting_name: s += '.{}'.format(setting_name)

        self.logger.debug('Load "%s" as the setting module', s)
        return __import__(s, fromlist=[''])


    def find_settings(self):
        """Looking for multiple settings"""
        settings = []
        if os.path.isfile(os.path.join(self.folder, self.project, 'settings.py')):
            settings.append(''.format(self.project))

        elif os.path.isdir(os.path.join(self.folder, self.project, 'settings')):
            for setting in os.listdir(os.path.join(self.folder, self.project, 'settings')):
                if os.path.isfile(os.path.join(os.path.join(self.folder, self.project, 'settings'), setting)):
                    settings.append(os.path.splitext(setting)[0])

        return settings

    def install_requirement(self):
        """Check the settings for dependencies and install thay"""
        packages = []

        for app in self.settings.INSTALLED_APPS:
            if app in self.APP_PIP:
                packages.append(self.APP_PIP[app])

        if self.settings.DATABASES['default']['ENGINE'] == 'django.db.backends.postgresql':
            packages.append('psycopg2-binary')

        try:
            if self.settings.EMAIL_BACKEND == 'django.core.mail.backends.smtp.EmailBackend':
                packages.append('django_smtp_ssl')
        except:
            self.logger.exception('No email backendd')

        if packages:
            self.execute([
                'cd {}'.format(self.folder),
                'python3 -m venv venv',
                'source venv/bin/activate',
                'pip install --upgrade {}'.format(' '.join(packages)),
                'deactivate',
            ])

    def setup_db(self):
        """Setup the right DB"""
        config = self.settings.DATABASES['default']
        engine = config['ENGINE']

        if engine == 'django.db.backends.postgresql':
            self.run('postgresql')



from axju.core.tools import SmartCLI
from axju.worker.django import DjangoWorker

def main():
    cli = SmartCLI(DjangoWorker)
    cli.add_argument('--folder', type=str, help='The django project folder.')
    cli.add_argument('--setting', type=str, help='Change the default the settings.')
    cli.run()

if __name__ == '__main__':
    main()



import logging
import argparse

def setup_logger(verbose, file):
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setFormatter(logging.Formatter('%(asctime)s %(message)s'))
    if verbose:
        ch.setLevel(logging.DEBUG)
    else:
        ch.setLevel(logging.INFO)
    logger.addHandler(ch)

    if file:
        fh = logging.FileHandler(file)
        fh.setFormatter(logging.Formatter('%(asctime)s %(levelname)-8s %(message)s'))
        fh.setLevel(logging.DEBUG)
        logger.addHandler(fh)

    return logger


def setup_parser():
    parser = argparse.ArgumentParser(description='Some atomation stuff, have fun')
    parser.add_argument('-v', '--verbose', action='store_true', help='For debugging ;)')
    parser.add_argument('-l', '--log', type=str, help='Logfile')
    parser.add_argument('-s', '--steps', nargs='+', type=str, default=['run'], help='All the steps')
    parser.add_argument('--json', type=argparse.FileType('r'), help='Load the steps json-file')
    parser.add_argument('--show', action='store_true', help='Show the steps.')

    return parser


class SmartCLI(object):
    """This only works, if the argument are for the init. There is no support
    for None."""

    def __init__(self, cls):
        self.cls = cls
        self.parser = setup_parser()
        self.additional = []

    def add_argument(self, *args, **kwargs):
        action = self.parser.add_argument(*args, **kwargs)
        self.additional.append(action.dest)

    def run(self):
        args = self.parser.parse_args()
        setup_logger(args.verbose, args.log)

        kwargs = {}
        for arg, value in vars(args).items():
            if arg in self.additional and value:
                kwargs[arg] = value

        worker = self.cls(**kwargs)

        if args.show: return worker.show(args.show)

        for step in args.steps: worker.run(step)



import os
from setuptools import setup, find_packages

base_dir = os.path.dirname(__file__)

def readme():
    with open('README.rst') as f:
        return f.read()

about = {}
with open(os.path.join(base_dir, 'axju', '__about__.py')) as f:
    exec(f.read(), about)

setup(
    name=about['__title__'],
    version=about['__version__'],
    description=about['__summary__'],
    url=about['__url__'],
    author=about['__author__'],
    author_email=about['__email__'],
    license=about['__license__'],

    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    long_description=readme(),
    include_package_data=True,
    zip_safe=False,

    install_requires=[
        'jinja2',
    ],

    entry_points = {
        'console_scripts': [
            'axju=axju.__main__:main',
            'axju-django=axju.worker.django.__main__:main',
        ],
    },

    keywords='git django deploying devops',
    classifiers=[
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)
